#include <QtCore>
#include <bundleListFileIO.h>
#include <readFile.h>

BundleListFileIO::BundleListFileIO(QHash<int,int> *h, QDir storageDir, QString fileName)
{
	hash = h;
	file = new QFile(storageDir.filePath(fileName));
	ReadFile conf;
	fileOption=conf.getUseFileHash();
	int WRITETIMER =conf.getWriteToFileTimer();
	if(fileOption==1)
	{
		timer = new QTimer();
		connect(timer, SIGNAL(timeout()), this, SLOT(writeAll()));
		timer->start(WRITETIMER);
	}

}

int BundleListFileIO::writeItem(int key)
{
	if(file->open(QIODevice::WriteOnly |QIODevice::Append| QIODevice::Text))
	{
		//add the key
		QString s = QString("%1").arg((int)key);
		//a tab to make things look nice
		s.append("\t");
		//add the values of the key
		QList<int> l = hash->values(key);
		for(int i=0; i<l.size(); i++)
		{
			s.append(QString("%1").arg((int)l.at(i)));
			s.append(";");	//we use ; as separator
		}
		s.remove(s.size()-1,1); //remove the last ;
		s.append("\n");
		file->write(s.toAscii());
		file->close();
		return 0;
	}else
	{
		//file IO error
		return -1;
	}
}

int BundleListFileIO::writeAll()
{
	//erase anything in the file
	file->open(QIODevice::WriteOnly |QIODevice::Truncate| QIODevice::Text);
	file->close();
	//then write each key and its values
	//QList<int> l = hash->uniqueKeys(); //!!This method doesn't exist
	QList<int> keys = hash->keys();
	QList<int> uniqueKeys;
	for(int i=0; i<keys.size(); i++)
	{
		if(!uniqueKeys.contains(keys.at(i)))
		{
			uniqueKeys.append(keys.at(i));
		}
	}
	int ret = 0;
	for(int i=0; i<uniqueKeys.size(); i++)
	{
		int status = writeItem(uniqueKeys.at(i));
		if(status < 0)
			ret = status;
	}
	return ret;
}

int BundleListFileIO::readAll()
{
	if (file->open(QIODevice::ReadOnly|QIODevice::Text))
	{
		QString line = file->readLine();
		while(line.size() > 0)
		{
			bool ok;
			QRegExp sep("[\t ]+");	//all tabs and spaces
			int key = line.section(sep, 0, 0).toInt(&ok, 10);
			if(!ok)
			{
				file->close();
				return -2;
			}
			QString values = line.section(sep, -1, -1);
			QStringList valuesList = values.split(";");
			for(int i=0; i<valuesList.size(); i++)
			{
				int value = valuesList.at(i).toInt();
				if(!ok)
				{
					file->close();
					return -2;
				}
				hash->insertMulti(key, value);
			}
			line = file->readLine();
		}
		file->close();
		return 0;
	}else
	{
		//file IO error
		return -1;
	}
}

